import 'package:cs_resources/generated/assets.dart';
import 'package:domain/entity/newsfeed_following_entity.dart';
import 'package:flutter/cupertino.dart';
import 'package:plugin_platform/engine/toast/toast_engine.dart';
import 'package:riverpod_annotation/riverpod_annotation.dart';
import 'package:router/ext/auto_router_extensions.dart';
import 'package:shared/utils/ext_dart.dart';
import 'package:shared/utils/log_utils.dart';
import 'package:widgets/load_state_layout.dart';
import 'package:widgets/widget_export.dart';

import '../../../respository/common_newsfeed.dart';
import '../../../router/page/community_page_router.dart';
import '../community_pageview_idx_data.dart';
import '../community_vm.dart';
import 'following_state.dart';

part 'following_vm.g.dart';

@riverpod
class FollowingVm extends _$FollowingVm {
  late CommonNewsFeedRespository commonNewsFeedRespositoryInstance;
  int _page = 1;  // 当前页
  int _limit = 10; // 每页数量
  int _count = 0; // 总条数
  
  bool _needShowPlaceholder = false; //是否展示LoadingView
  // Refresh 控制器
  final EasyRefreshController refreshController = EasyRefreshController(
    controlFinishRefresh: true,  //允许刷新
    controlFinishLoad: true,   //允许加载
  );

  FollowingState initState() {
    return FollowingState(
      list: []
    );
  }

  @override
  FollowingState build(){
    // 引入数据仓库
    commonNewsFeedRespositoryInstance = ref.read(commonNewsFeedRespositoryProvider);
    final state = initState();
    Log.d("---------------following-----------build---------------------");

    return state;
  }

  //刷新页面状态
  void changeLoadingState(LoadState loadState, String? errorMsg) {
    state = state.copyWith(
        loadingState: loadState,
        errorMessage: errorMsg
    );
  }

  // 初始化页面数据
  initPageData() {
    Log.d("----following_vm-----initPageData   ${state.loadingState}");
    onRefresh();
  }

  // 上拉加载 更多
  Future loadMore() async {
    bool isShowing = await ref.read(communityVmProvider.notifier).isCurrentPageViewShowing(CommunityPageViewIdxData.following);
    if(isShowing){
      Log.d("----following_vm-----loadMore");
      // await Future.delayed(const Duration(seconds: 2));
      // if(state.list.length >= _count){
      //   return;
      // }else {
      //   int page = _page + 1;
      //   state = state.copyWith(page: page,);
      //   getListData();
      // }
      _page++;
      getListData();
    }else {
      refreshController.finishLoad();
    }
  }


  // 下拉刷新
  Future onRefresh() async {
    Log.d("--following--following_vm-----onRefresh ");
    bool isShowing = await ref.read(communityVmProvider.notifier).isCurrentPageViewShowing(CommunityPageViewIdxData.following);
    if(isShowing){
      // 当前pageView 页面正处于显示状态
      Log.d("----following_vm-----onRefresh ");
      // await Future.delayed(const Duration(seconds: 2));
      _page = 1;
      getListData();
    }else {
      refreshController.finishRefresh();
    }
  }

  // 重试请求
  Future retryRequest() async {
    bool isShowing = await ref.read(communityVmProvider.notifier).isCurrentPageViewShowing(CommunityPageViewIdxData.following);
    if(isShowing){
      _page = 1;
      _needShowPlaceholder = true;
      getListData();
    }else {
      refreshController.finishRefresh();
    }
  }


  // 获取list 列表数据
  Future getListData<T>() async {
    if (_needShowPlaceholder) {
      changeLoadingState(LoadState.State_Loading, null);
    }
    Log.d("加载listData数据---------------start--${_page}---");

    try {
      //请求网络
      Map<String, dynamic>  params = {
        "page": _page,
        "limit": _limit,
      };
      Log.d("请求参数------$params");
      final result = await commonNewsFeedRespositoryInstance.fetchFollowingList(params);
      //校验成功失败
      if (result.isSuccess) {
        handlerResultList((result.data  as NewsfeedFollowingEntity).list);
      } else {
        String errorMessage = result.errorMsg!;
        changeLoadingState(LoadState.State_Error, errorMessage);
        ToastEngine.show(result.errorMsg ?? "Network Load Error");
      }
    } catch (e) {
      ToastEngine.show("Error: $e");
    }
    // // 最后赋值
    // _needShowPlaceholder = false;
  }


  void handlerResultList(List<NewsfeedFollowingList>? list) {
    if (list != null && list.isNotEmpty) {
      //有数据，判断是刷新还是加载更多的数据
      if (_page == 1) {
        //刷新的方式
        state.list!.clear();
        state.list!.addAll(list.map((item) => item.toJson()).toList());
        refreshController.finishRefresh();

        //更新展示的状态
        changeLoadingState(LoadState.State_Success, null);
      } else {
        //加载更多
        final allList = state.list;
        state = state.copyWith(list: allList);
        refreshController.finishLoad();
      }
    } else {
      if (_page == 1) {
        //展示无数据的布局
        state.list!.clear();
        changeLoadingState(LoadState.State_Empty, null);
        refreshController.finishRefresh();
      } else {
        //展示加载完成，没有更多数据了
        if(state.list!.length == 0){
          changeLoadingState(LoadState.State_Empty, null);
          refreshController.finishLoad();
        }else {
          if(_needShowPlaceholder){
            changeLoadingState(LoadState.State_Success, null);
          }
        }
        //更新展示的状态
        refreshController.finishLoad(IndicatorResult.noMore);
      }
    }
  }

  // 点赞/取消点赞
  Future handlerLikeClick(int id, bool isLike, int? itemidx) async {
    try {
      List<Map<String, dynamic>> listCopyDta =  List.from(state.list!);
      final result = await commonNewsFeedRespositoryInstance.fetchLikeClick({
        "id": id,
      });
      if (result.isSuccess) {
        if(itemidx != null){
          // 修改 listCopyDta[itemIdx] 中的 like 状态 和 likes_count
          listCopyDta![itemidx]['liked'] = !isLike;

          if(isLike){
            // 取消点赞
            if(listCopyDta![itemidx]['likes_count']>0){
              listCopyDta![itemidx]['likes_count'] = listCopyDta![itemidx]['likes_count'] - 1;
            }
          }else {
            listCopyDta![itemidx]['likes_count'] = listCopyDta![itemidx]['likes_count'] + 1;
          }
        }else {
          // 详情中的点赞 需要找到对应的 item 进行 修改 like 和 likes_count
          listCopyDta!.forEach((carditem) {
            if(carditem['id'] == id){
              carditem['liked'] = !isLike;
              if(isLike){
                // 取消点赞
                if(carditem['likes_count']>0){
                  carditem['likes_count'] = carditem['likes_count'] - 1;
                }
              }else {
                carditem['likes_count'] = carditem['likes_count'] + 1;
              }
            }
          });
        }

        state = state.copyWith(list: listCopyDta);

        final String toastMsg = isLike ? "Cancel successfully": "Liked successfully";
        ToastEngine.show(toastMsg);

        return true;
      }else {
        return false;
      }
    }catch(error) {
      return false;
    }
  }

  // 点击 like comments  share
  Future<bool?> handlerClickActionBtn(String? actionStr, Map<String, dynamic> item, int itemidx) async{
    final id = item['id'];
    final liked = item.getValue('liked', false);
    switch (actionStr) {
      case 'like':
        return await handlerLikeClick(id, liked, itemidx);
      case 'comments':
        Log.d("点击了 评论");
        handlerGotoDetail(null, id);
        break;
      case 'share':
        Log.d("点击了 分享");
        handlerGotoDetail(null, id);
        break;
      default:
        Log.d("点击了卡片");
        handlerGotoDetail(null, id);
        break;
    }
  }
  // 关注/取消关注
  Future<bool> handlerFollow(BuildContext? context, int to_user_id,int cardId, bool isFollow) async{
    Log.d("点击了 关注");
    try {
      final result = await commonNewsFeedRespositoryInstance.handlerFollowOrCancel({
        "to_user_id": to_user_id,
      });
      if(result.isSuccess){
        // 修改cardId 对应的 card item 的 account 里面的 followed
        if(cardId!=null){
          final listCopyDta = List<Map<String, dynamic>>.from(state.list!);
          listCopyDta!.forEach((carditem) {
            if(carditem['id'] == cardId){
              carditem['account']['followed'] = !carditem['account']['followed'];
            }
          });
          state = state.copyWith(list: listCopyDta);
        }
        return true;
      }else {
        return false;
      }
    }catch(error){
      Log.d("error: $error");
      return false;
    }
  }

  // 去详情页面
  void handlerGotoDetail(BuildContext? context, int id){
    Log.d("去详情页面");
    appRouter.push(NewsfeedDetailPageRoute(id: id, type:'following'));
  }
}