import 'package:cpt_notice_board/modules/notice_board/page/notice_board_page.dart';
import 'package:cs_resources/theme/app_colors_theme.dart';
import 'package:flutter/material.dart';
import 'package:auto_route/auto_route.dart';
import 'package:hooks_riverpod/hooks_riverpod.dart';
import 'package:router/ext/auto_router_extensions.dart';
import 'package:shared/utils/log_utils.dart';
import 'package:shared/utils/color_utils.dart';
import 'package:widgets/ext/ex_widget.dart';
import 'package:widgets/my_appbar.dart';
import 'package:widgets/my_load_image.dart';
import 'package:widgets/widget_export.dart';
import 'package:cs_resources/generated/assets.dart';

import '../../../router/page/notice_board_page_router.dart';
import '../vm/documents_list_vm.dart';

@RoutePage()
class DocumentsListPage extends HookConsumerWidget {
  const DocumentsListPage({Key? key}) : super(key: key);

  //启动当前页面
  static void startInstance({BuildContext? context}) {
    if (context != null) {
      context.router.push(const DocumentsListPageRoute());
    } else {
      appRouter.push(const DocumentsListPageRoute());
    }
  }

  Widget _buildItemLeftSection(BuildContext context, WidgetRef ref, item, _vm) {
    return Container(
      // color: Colors.blue,
      child: Text(
        item['title'],
        maxLines: 2, // 设置最大行数为2
        overflow: TextOverflow.ellipsis, // 超出部分用省略号表示
        style: const TextStyle(
            fontSize: 16.0,
            color: Colors.black,
            fontWeight: FontWeight.w700), // 设置字体大小
      ),
    ).marginOnly(right: 17.5);
  }

  Widget _buildItemRightSection(
      BuildContext context, WidgetRef ref, item, _vm) {
    return Container(
      color: Colors.white,
      child: TextButton(
        onPressed: () {},
        style: TextButton.styleFrom(
          foregroundColor: Colors.black,
          backgroundColor: ColorUtils.string2Color('#4161D0'), // 背景颜色
          minimumSize: const Size(91.5, 30), // 最小宽度和高度
          padding:
              const EdgeInsets.symmetric(horizontal: 11.0, vertical: 9), // 内边距
          shape: RoundedRectangleBorder(
            borderRadius: BorderRadius.circular(5), // 圆角
            side: BorderSide(
              color: ColorUtils.string2Color('#4161D0'),
              width: 1.0,
            ), // 边框
          ),
        ),
        child: const Text('Open Folder',
          style: const TextStyle(color: Colors.white,),
        ),
      ),
    );
  }

  // listitem
  Widget _buildSaleItem(BuildContext context, WidgetRef ref, item, _vm) {
    return Container(
      decoration: const BoxDecoration(
        color: Colors.white,
        borderRadius: BorderRadius.all(Radius.circular(6.0)),
        boxShadow: [
          BoxShadow(color: Color.fromRGBO(184, 191, 217, 0.3), blurRadius: 6)
        ],
      ),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.center,
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisSize: MainAxisSize.max,
        children: [
          Container(
            width: MediaQuery.of(context).size.width - 30,
            height: 70,
            child: Row(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.center,
              children: [
                Expanded(child: _buildItemLeftSection(context, ref, item, _vm)),
                Container(
                  // width: 120,
                  // child: _buildItemRightSection(context, ref, item, _vm),
                  // child: TextButton(onPressed: (){}, child: Text("fdsfds")),
                ),
              ],
            ).paddingOnly(left: 20,right: 20),
          ).constrained(
            minHeight: 70,
          ),
        ],
      ).onTap(() {
        // 去详情
        _vm.goNewsDetail(item);
      }),
    ).marginOnly(left: 15, bottom: 15, right: 15);
  }

  // list
  Widget _buildSaleList(BuildContext context, WidgetRef ref, _vm) {
    List itemsList = _vm.state.list.toList();
    return ListView.builder(
      itemCount: itemsList.length,
      itemBuilder: (context, index) {
        return _buildSaleItem(context, ref, itemsList[index], _vm);
      },
    );
  }

  @override
  Widget build(BuildContext context, WidgetRef ref) {
    final _vm = ref.read(documentsListVmProvider.notifier);

    return Scaffold(
      appBar: MyAppBar.appBar(
        context,
        "Documents",
        backgroundColor: context.appColors.backgroundWhite,
      ),
      body: Container(
          child: EasyRefresh(
        // 上拉加载
        onLoad: () async {
          Log.d("----onLoad");
          _vm.onLoadData();
        },
        // 下拉刷新
        onRefresh: () async {
          Log.d("----onRefresh");
          _vm.refreshListData();
        },
        child: Container(
            color: ColorUtils.string2Color('#F2F3F6'),
            padding: const EdgeInsets.only(top: 15),
            child: _buildSaleList(context, ref, _vm)),
      )),
    );
  }
}
